@@ -24,6 +24,7 @@ __all__ = [
     'ParamConflict',
     'ParamsMissing',
     'NoActiveContext',
+    'InputValidationError',
 ]
 
 
@@ -158,6 +159,10 @@ class NoActiveContext(CommandParserException, RuntimeError):
     """Raised when attempting to perform an action that requires an active context while no context is active."""
 
 
+class InputValidationError(CommandParserException, ValueError):
+    """Raised when a custom Input's conversion/validation fails"""
+
+
 # endregion
 
 # region Internal Exceptions
