@@ -2,9 +2,9 @@
 :author: Doug Skrypa
 """
 
-from typing import Union, Sequence, Tuple, Set
+from typing import Union, Optional, Sequence, Tuple, Set
 
-NargsValue = Union[str, int, Tuple[int, int], Sequence[int], Set[int], range]
+NargsValue = Union[str, int, Tuple[int, Optional[int]], Sequence[int], Set[int], range]
 
 NARGS_STR_RANGES = {'?': (0, 1), '*': (0, None), '+': (1, None)}
 SET_ERROR_FMT = 'Invalid nargs={!r} set - expected non-empty set where all values are integers >= 0'
